/*
  Часы с отображением температуры и влажности с индикацией на VFD (ВЛИ) дисплее покупателя 20*2 Futaba (vfd9cb1010 rev 1.2)
  AMatroskin 2020 год.
*/

#include <GyverButton.h>
#include <SoftwareSerial.h>
#include <Wire.h>
#include <iarduino_RTC.h>
#include <LiquidCrystal_I2C.h>
#include <Adafruit_BME280.h>

#define SEALEVELPRESSURE_HPA (1010.7)
#define BTN1 17 //____________________________________________________Пины кнопок
#define BTN2 16
#define BTN3 15

#define COR_TEMP 9     //_____________________________________________Коректирвка температуры (вычитается из показаний датчика)
#define FIRST_LEVEL 8 //_____________________________________________Уровни переключения яркости
#define SECOND_LEVEL 150
#define THIRD_LEVEL  210
#define SHIFT 1  //___________________________________________________Изменение порядка отображения на экране время -> температура/влажность (дата, день недели) и наоборот происходит раз в минуту
//____________________________________________________________________1 -автоматическая перестановка включена, 0 - выключена. (Уменьшение неравномерности износа люминофора)

Adafruit_BME280 bme; //______________________________________________Объекты датчика температуры/влажности, часов RTC, последовательного порта и кнопок
iarduino_RTC watch(RTC_DS3231);
SoftwareSerial mySerial(11, 10);
GButton butt1(BTN1);
GButton butt2(BTN2);
GButton butt3(BTN3);

int temp_round, temp_round_c1, temp_round_c2, temp_round_d; //_______Переменные для отображения температуры
int hum_round, hum_round_c1, hum_round_c2, hum_round_d;  //__________Переменные для отображения влажности
uint32_t counter = 1000, counter2 = 0; //____________________________Переменные счетчиков

int val_filter; //___________________________________________________Переменные для фильтра датчика освещенности
long average;
int valArray[10];
byte idx = 0;

byte index, min_hour = 0; //_________________________________________Служебные переменные
bool flag = 1, flag2 = 0, set_view = 0;

String up_half [12] { //_____________________________________________Верхние половины цифр
  "\260\261", // ноль
  "\264\265", // один
  "\270\271", // два
  "\274\275", // три
  "\300\301", // четыре
  "\304\305", // пять
  "\310\311", // шесть
  "\314\315", // семь
  "\320\321", // восемь
  "\324\325", // девять
  "\330",     // :
};

String down_half [12] { //___________________________________________Нижние половины цифр
  "\262\263", // ноль
  "\266\267", // один
  "\272\273", // два
  "\276\277", // три
  "\302\303", // четыре
  "\306\307", // пять
  "\312\313", // шесть
  "\316\317", // семь
  "\322\323", // восемь
  "\326\327", // девять
  "\331",     // :
};

void setup() {
  delay(3000); //____________тут надо подождать
  pinMode(10, OUTPUT);
  watch.begin();
  bme.begin(0x76);
  mySerial.begin(19200);
  Serial.begin(9600);
  mySerial.write(0x1B); //
  mySerial.write(0x40); //____инициализация дисплея
  delay(1000); //_____________и тут надо подождать
  CustomFontLoad(); //________загрузка своего шрифта
  delay(2000); //_____________и тут надо подождать
  mySerial.write(0x0b); //____курсор домой
  get_temp (); //_____________получаем температуру и влажность
  show_time (); //____________выводим информацию на дисплей

  for (byte i; i < 10; i++) { //_Набираем массив значений с датчика освещенности для корректной работы сразу после запуска
    Light (analogRead(20)); //___Датчик света подключен к 20 пину (А6)
  }
}

void loop() {
  butt1.tick();
  if (butt1.isHolded() && butt1.isHold()) { //__Переключение режимов отображения/настройки времени
    set_view = ! set_view;
    mySerial.write(0x0b); //____________________курсор домой
  }
  switch (set_view) {
    case 0:   //________________________________Выводим время и устанавливаем яркость экрана раз в секунду
      min_hour = 0;
      if (millis() - counter >= 1000) {
        show_time ();
        flag = ! flag;
        counter += 1000;
        if ((Light (analogRead(20)) > THIRD_LEVEL)) {
          set_brightness(4);
        }
        if (((Light (analogRead(20)) <= THIRD_LEVEL)) && ((Light (analogRead(20))) > SECOND_LEVEL) ) {
          set_brightness(3);
        }
        if (((Light (analogRead(20)) <= SECOND_LEVEL)) && ((Light (analogRead(20)) > FIRST_LEVEL) )) {
          set_brightness(2);
        }
        if (Light (analogRead(20)) <= FIRST_LEVEL)  {
          set_brightness(1);
        }
      }
      break;

    case 1: //________________________________Устанавливаем время
      butt2.tick();
      butt3.tick();
      if (butt1.isClick()) {
        min_hour ++;
      }
      if (min_hour > 5) min_hour = 0;
      if (millis() - counter >= 200) {
        counter += 200;
        set_time();
      }
      break;
  }

  if (millis() - counter2 >= 5000) { //_______получаем температуру и влажность раз в 5 секунд
    counter2 += 5000;
    get_temp();
  }
}

void CustomFontLoad() { //_____________________________Создаем свои символы
  mySerial.write(0x1B);
  mySerial.write(0x25);
  mySerial.write(0x01); // Разрешаем кастомные символы

  mySerial.write(0x1B);
  mySerial.write(0x26);
  mySerial.write(0x01); // Начинаем запись кастомных символов
  mySerial.write(0xB0); // Первый адрес диапазона
  mySerial.write(0xdb); // Последний адрес диапазона

  //_________________________Создание символов

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x1F); // Большой ноль 1
  mySerial.write(0x3F); // \260
  mySerial.write(0X70);
  mySerial.write(0x60);
  mySerial.write(0x60);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x63); // Большой ноль 2
  mySerial.write(0x67); // \261
  mySerial.write(0x76);
  mySerial.write(0x3F);
  mySerial.write(0x1F);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x7C); // Большой ноль 3
  mySerial.write(0x7E); // \262
  mySerial.write(0x37);
  mySerial.write(0x73);
  mySerial.write(0x63);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x3); // Большой ноль 4
  mySerial.write(0x3); // \263
  mySerial.write(0x7);
  mySerial.write(0x7E);
  mySerial.write(0x7C);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Единица 1
  mySerial.write((int)0x00); // \264
  mySerial.write((int)0x00);
  mySerial.write(0x18);
  mySerial.write(0x38);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x7F); // Единица 2
  mySerial.write(0x7F); // \265
  mySerial.write((int)0x00);
  mySerial.write((int)0x00);
  mySerial.write((int)0x00);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Единица 3
  mySerial.write((int)0x00); // \266
  mySerial.write((int)0x00);
  mySerial.write(0x3);
  mySerial.write(0x3);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x7F); // Единица 4
  mySerial.write(0x7F); // \267
  mySerial.write(0X3);
  mySerial.write(0x3);
  mySerial.write((int)0x00);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Двойка 1
  mySerial.write(0x30); // \270
  mySerial.write(0X70);
  mySerial.write(0x60);
  mySerial.write(0x60);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x60); // Двойка 2
  mySerial.write(0x61); // \271
  mySerial.write(0X73);
  mySerial.write(0x3F);
  mySerial.write(0x1E);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x3); // Двойка 3
  mySerial.write(0x7); // \272
  mySerial.write(0XF);
  mySerial.write(0x1F);
  mySerial.write(0x3B);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x73); // Двойка 4
  mySerial.write(0x63); // \273
  mySerial.write(0X43);
  mySerial.write(0x3);
  mySerial.write(0x3);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Тройка 1
  mySerial.write(0x60); // \274
  mySerial.write(0X60);
  mySerial.write(0x61);
  mySerial.write(0x63);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x67); // Тройка 2
  mySerial.write(0x6E); // \275
  mySerial.write(0X7C);
  mySerial.write(0x78);
  mySerial.write(0x70);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0xC); // Тройка 3
  mySerial.write(0xE); // \276
  mySerial.write(0X7);
  mySerial.write(0x43);
  mySerial.write(0x43);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x43); // Тройка 4
  mySerial.write(0x43); // \277
  mySerial.write(0X67);
  mySerial.write(0x7E);
  mySerial.write(0x3C);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x1); // Четверка 1
  mySerial.write(0x3); // \300
  mySerial.write(0X7);
  mySerial.write(0xE);
  mySerial.write(0x1C);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x60); // Четверка 2
  mySerial.write(0x7F); // \301
  mySerial.write(0X7F);
  mySerial.write((int)0x00);
  mySerial.write((int)0x00);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x70); // Четверка 3
  mySerial.write(0x70); // \302
  mySerial.write(0X30);
  mySerial.write(0x30);
  mySerial.write(0x30);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x30); // Четверка 4
  mySerial.write(0x7F); // \303
  mySerial.write(0X7F);
  mySerial.write(0x30);
  mySerial.write(0x30);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Пятерка 1
  mySerial.write(0x7F); // \304
  mySerial.write(0X7F);
  mySerial.write(0x63);
  mySerial.write(0x63);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x63); // Пятерка 2
  mySerial.write(0x63); // \305
  mySerial.write(0X63);
  mySerial.write(0x61);
  mySerial.write(0x60);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0xC); // Пятерка 3
  mySerial.write(0xE); // \306
  mySerial.write(0X7);
  mySerial.write(0x3);
  mySerial.write(0x3);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x3); // Пятерка 4
  mySerial.write(0x3); // \307
  mySerial.write(0X47);
  mySerial.write(0x7E);
  mySerial.write(0x7C);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0xF); // Шестерка 1
  mySerial.write(0x1F); // \310
  mySerial.write(0X39);
  mySerial.write(0x73);
  mySerial.write(0x63);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x63); // Шестерка 2
  mySerial.write(0x63); // \311
  mySerial.write(0X63);
  mySerial.write(0x61);
  mySerial.write((int)0x00);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x7C); // Шестерка 3
  mySerial.write(0x7E); // \312
  mySerial.write(0X47);
  mySerial.write(0x43);
  mySerial.write(0x3);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x3); // Шестерка 4
  mySerial.write(0x3); // \313
  mySerial.write(0X47);
  mySerial.write(0x7E);
  mySerial.write(0x7C);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Семерка 1
  mySerial.write(0x60); // \314
  mySerial.write(0X60);
  mySerial.write(0x60);
  mySerial.write(0x60);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x63); // Семерка 2
  mySerial.write(0x67); // \315
  mySerial.write(0X6E);
  mySerial.write(0x7C);
  mySerial.write(0x78);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Семерка 3
  mySerial.write((int)0x00); // \316
  mySerial.write((int)0x00);
  mySerial.write(0x3F);
  mySerial.write(0x7F);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Семерка 4
  mySerial.write((int)0x00); // \317
  mySerial.write((int)0x00);
  mySerial.write((int)0x00);
  mySerial.write((int)0x00);


  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x1E); // Восьмерка 1
  mySerial.write(0x3F); // \320
  mySerial.write(0X73);
  mySerial.write(0x61);
  mySerial.write(0x61);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x61); // Восьмерка 2
  mySerial.write(0x61); // \321
  mySerial.write(0X73);
  mySerial.write(0x3F);
  mySerial.write(0x1E);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x3C); // Восьмерка 3
  mySerial.write(0x7E); // \322
  mySerial.write(0X67);
  mySerial.write(0x43);
  mySerial.write(0x43);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x43); // Восьмерка 4
  mySerial.write(0x43); // \323
  mySerial.write(0X67);
  mySerial.write(0x7E);
  mySerial.write(0x3C);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x1F); // Девятка 1
  mySerial.write(0x3F); // \324
  mySerial.write(0X71);
  mySerial.write(0x60);
  mySerial.write(0x60);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x60); // Девятка 2
  mySerial.write(0x60); // \325
  mySerial.write(0X71);
  mySerial.write(0x3F);
  mySerial.write(0x1F);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Девятка 3
  mySerial.write(0x43); // \326
  mySerial.write(0X63);
  mySerial.write(0x63);
  mySerial.write(0x63);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x63); // Девятка 4
  mySerial.write(0x67); // \327
  mySerial.write(0X4E);
  mySerial.write(0x7C);
  mySerial.write(0x78);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Двоеточие верх
  mySerial.write(0xE); // \330
  mySerial.write(0XE);
  mySerial.write(0xE);
  mySerial.write((int)0x00); //

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write((int)0x00); // Двоеточие низ
  mySerial.write(0x38); // \331
  mySerial.write(0X38);
  mySerial.write(0x38);
  mySerial.write((int)0x00);

  mySerial.write(0x05); // Размер символа 5 байт
  mySerial.write(0x30); // Значок градуса °
  mySerial.write(0x48); // \332
  mySerial.write(0x48);
  mySerial.write(0x30);
  mySerial.write((int)0x00);
}

void show_time () { //_________________________________Отображение времени
  watch.gettime();
  byte h1, h2, m1, m2, s1, s2, d1, d2, mon1, mon2;
  String dots_up, dots_down, days;
  if (flag) { //_______________________________________Мигаем точками
    dots_up = up_half [10];
    dots_down = down_half [10];
  } else {
    dots_up = " ";
    dots_down = " ";
  }
  h1 = watch.Hours / 10;
  h2 = watch.Hours % 10;
  m1 = watch.minutes / 10;
  m2 = watch.minutes % 10;
  s1 = watch.seconds / 10;
  s2 = watch.seconds % 10;
  d1 = watch.day / 10;
  d2 = watch.day % 10;
  mon1 = watch.month / 10;
  mon2 = watch.month % 10;
  days = watch.gettime("D");

  //_____________________________________________________Отрисовка содержимого экрана
  if ((SHIFT) && ((m2 % 2) != 0)) { //_____________________Если изменение порядка отображения на экране активно
    mySerial.print(up_half [h1]); //_______________________Первая строка: верхние половины больших цифр, точек, значение температуры, символ градуса
    mySerial.print(up_half [h2]);
    mySerial.print(dots_up);
    mySerial.print(up_half [m1]);
    mySerial.print(up_half [m2]);
    mySerial.print(dots_up);
    mySerial.print(up_half [s1]);
    mySerial.print(up_half [s2]);
    mySerial.print(" ");

    /*mySerial.print(" "); //Дата справа
      mySerial.print(days);
      mySerial.print(" ");*/

    mySerial.print(temp_round_c1);
    mySerial.print(temp_round_c2);
    mySerial.print(".");
    mySerial.print(temp_round_d);
    mySerial.print("\332");
    mySerial.print(down_half [h1]);  //_____________________Вторая строка: нижние половины больших цифр, точек, влажность
    mySerial.print(down_half [h2]);
    mySerial.print(dots_down);
    mySerial.print(down_half [m1]);
    mySerial.print(down_half [m2]);
    mySerial.print(dots_down);
    mySerial.print(down_half [s1]);
    mySerial.print(down_half [s2]);
    mySerial.print(" ");

    /*mySerial.print(d1);//Дата справа
      mySerial.print(d2);
      mySerial.print("/");
      mySerial.print(mon1);
      mySerial.print(mon2);  */

    mySerial.print(hum_round_c1);
    mySerial.print(hum_round_c2);
    mySerial.print(".");
    mySerial.print(hum_round_d);
    mySerial.print("%");
  } else if ((SHIFT) && ((m2 % 2) == 0)) {

    mySerial.print(" "); //Дата слева
    mySerial.print(days);
    mySerial.print(" ");

    /*mySerial.print(temp_round_c1);
      mySerial.print(temp_round_c2);
      mySerial.print(".");
      mySerial.print(temp_round_d);
      mySerial.print("\332");*/

    mySerial.print(" ");
    mySerial.print(up_half [h1]); //_______________________Первая строка: верхние половины больших цифр, точек, значение температуры, символ градуса
    mySerial.print(up_half [h2]);
    mySerial.print(dots_up);
    mySerial.print(up_half [m1]);
    mySerial.print(up_half [m2]);
    mySerial.print(dots_up);
    mySerial.print(up_half [s1]);
    mySerial.print(up_half [s2]);

    mySerial.print(d1); //Дата слева
    mySerial.print(d2);
    mySerial.print("/");
    mySerial.print(mon1);
    mySerial.print(mon2);
    
    /*mySerial.print(hum_round_c1);
      mySerial.print(hum_round_c2);
      mySerial.print(".");
      mySerial.print(hum_round_d);
      mySerial.print("%");*/

    mySerial.print(" ");
    mySerial.print(down_half [h1]);  //_____________________Вторая строка: нижние половины больших цифр, точек, влажность
    mySerial.print(down_half [h2]);
    mySerial.print(dots_down);
    mySerial.print(down_half [m1]);
    mySerial.print(down_half [m2]);
    mySerial.print(dots_down);
    mySerial.print(down_half [s1]);
    mySerial.print(down_half [s2]);
  } else { //______________________________________________Если изменение порядка отображения на экране не активно
    mySerial.print(up_half [h1]); //_______________________Первая строка: верхние половины больших цифр, точек, значение температуры, символ градуса
    mySerial.print(up_half [h2]);
    mySerial.print(dots_up);
    mySerial.print(up_half [m1]);
    mySerial.print(up_half [m2]);
    mySerial.print(dots_up);
    mySerial.print(up_half [s1]);
    mySerial.print(up_half [s2]);
    mySerial.print(" ");
    mySerial.print(temp_round_c1);
    mySerial.print(temp_round_c2);
    mySerial.print(".");
    mySerial.print(temp_round_d);
    mySerial.print("\332");
    mySerial.print(down_half [h1]);  //_____________________Вторая строка: нижние половины больших цифр, точек, влажность
    mySerial.print(down_half [h2]);
    mySerial.print(dots_down);
    mySerial.print(down_half [m1]);
    mySerial.print(down_half [m2]);
    mySerial.print(dots_down);
    mySerial.print(down_half [s1]);
    mySerial.print(down_half [s2]);
    mySerial.print(" ");
    mySerial.print(hum_round_c1);
    mySerial.print(hum_round_c2);
    mySerial.print(".");
    mySerial.print(hum_round_d);
    mySerial.print("%");
  }
}

void get_temp() { //___________________________________Получение данных о температуре и влажности и их обработка (округление, разбиение на разряды)
  temp_round = bme.readTemperature() - COR_TEMP;
  temp_round_c1 = (temp_round / 10);
  temp_round_c2 = (temp_round % 10);
  int d  = (bme.readTemperature() * 10);
  temp_round_d = (d % 10);

  hum_round = bme.readHumidity();
  hum_round_c1 = (hum_round / 10);
  hum_round_c2 = (hum_round % 10);
  int f  = (bme.readHumidity() * 10);
  hum_round_d = (d % 10);
}

void set_brightness(byte lev) {//______________________Установка яркости дисплея (1 - 4)
  byte level = lev;
  switch (level) {
    case 1:
      mySerial.write(0x1B); //_________________________Первый уровень яркости
      mySerial.write(0x2A);
      mySerial.write(0x01);
      break;
    case 2:
      mySerial.write(0x1B); //_________________________Второй уровень яркости
      mySerial.write(0x2A);
      mySerial.write(0x02);
      break;
    case 3:
      mySerial.write(0x1B); //_________________________Третий уровень яркости
      mySerial.write(0x2A);
      mySerial.write(0x03);
      break;
    case 4:
      mySerial.write(0x1B); //_________________________Четвертый уровень яркости
      mySerial.write(0x2A);
      mySerial.write(0x04);
      break;
  }
}

void set_time() { //___________________________________Экран установки времени
  flag = ! flag;
  if (min_hour == 0) {
    mySerial.print("   Setting Hours    ");
  } else if (min_hour == 1) {
    mySerial.print("   Setting Minutes  ");
  } else if (min_hour == 2) {
    mySerial.print("   Reset Seconds    ");
  } else if (min_hour == 3) {
    mySerial.print("Setting Day of Week ");
  } else if (min_hour == 4) {
    mySerial.print("Setting Day of Month");
  } else if (min_hour == 5) {
    mySerial.print("   Setting Month    ");
  }

  watch.gettime();
  mySerial.print(watch.Hours / 10);
  mySerial.print(watch.Hours % 10);
  mySerial.print(":");
  mySerial.print(watch.minutes / 10);
  mySerial.print(watch.minutes % 10);
  mySerial.print(":");
  mySerial.print(watch.seconds / 10);
  mySerial.print(watch.seconds % 10);
  mySerial.print("  ");
  mySerial.print(watch.gettime("D"));
  mySerial.print(" ");
  mySerial.print(watch.day / 10);
  mySerial.print(watch.day % 10);
  mySerial.print(" ");
  mySerial.print(watch.gettime("M"));

  //__________________________________________Минуты
  if ((min_hour == 1) && butt2.isClick() ) {
    //flag2 = 0;
    int m = (watch.minutes + 1);
    if (m >= 60) m = 0;
    watch.settime(-1, m);
  }
  if ((min_hour == 1) && butt2.isHold() ) {
    int m = (watch.minutes + 5);
    if (m >= 60) m = 0;
    watch.settime(-1, m);
  }
  if ((min_hour == 1) && butt3.isClick()) {
    // flag2 = 0;
    int m = (watch.minutes - 1);
    if (m < 0) m = 59;
    watch.settime(-1, m);
  }
  if ((min_hour == 1) && butt3.isHold()) {
    int m = (watch.minutes - 5);
    if (m < 0) m = 59;
    watch.settime(-1, m);
  }

  //__________________________________________Часы
  if ((min_hour == 0) && butt2.isClick() ) {
    int m = (watch.Hours + 1);
    if (m >= 24) m = 0;
    watch.settime(-1, -1,  m);
  }
  if ((min_hour == 0) && butt2.isHold() ) {
    int m = (watch.Hours + 2);
    if (m >= 24) m = 0;
    watch.settime(-1, -1,  m);
  }
  if ((min_hour == 0) && butt3.isClick() ) {
    int m = (watch.Hours - 1);
    if (m < 0) m = 23;
    watch.settime(-1, -1,  m);
  }
  if ((min_hour == 0) && butt3.isHold() ) {
    int m = (watch.Hours - 2);
    if (m < 0) m = 23;
    watch.settime(-1, -1,  m);
  }
  //__________________________________________Секунды
  if ((min_hour == 2) && butt2.isClick()) {
    watch.settime(0);
  }
  if ((min_hour == 2) && butt3.isClick()) {
    watch.settime(0);
  }
  //__________________________________________День недели
  if ((min_hour == 3) && butt2.isClick()) {
    int m = (watch.weekday + 1);
    if (m > 6) m = 0;
    watch.settime(-1, -1, -1, -1, -1, -1, m);
  }
  if ((min_hour == 3) && butt3.isClick()) {
    int m = (watch.weekday - 1);
    if (m < 0) m = 6;
    watch.settime(-1, -1, -1, -1, -1, -1, m);
  }
  //__________________________________________День месяца
  if ((min_hour == 4) && butt2.isClick()) {
    int m = (watch.day + 1);
    if (m > 31) m = 1;
    watch.settime(-1, -1, -1, m);
  }
  if ((min_hour == 4) && butt3.isClick()) {
    int m = (watch.day - 1);
    if (m < 1) m = 31;
    watch.settime(-1, -1, -1, m);
  }
  //__________________________________________Месяц
  if ((min_hour == 5) && butt2.isClick()) {
    int m = (watch.month + 1);
    if (m > 12) m = 1;
    watch.settime(-1, -1, -1, -1, m);
  }
  if ((min_hour == 5) && butt3.isClick()) {
    int m = (watch.month - 1);
    if (m < 1) m = 12;
    watch.settime(-1, -1, -1, -1, m);
  }
}

//__________________________________________Фильтр значений для датчика света (среднее арифметическое из последних 10 значений)

uint8_t Light(int newVal) { //________________принимает новое значение
  valArray[idx] = newVal;  //_________________пишем каждый раз в новую ячейку
  if (++idx >= 10) idx = 0; //________________перезаписывая самое старое значение
  average = 0;             //_________________обнуляем среднее
  for (int i = 0; i < 10; i++) {
    average += valArray[i]; //________________суммируем
  }
  average /= 10;            //________________делим
  return average;           //________________возвращаем
}
